# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.msp.postgresql.v1alpha1``"""

import builtins as builtins
import collections.abc as abc
import datetime as datetime
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import google.protobuf.timestamp_pb2 as timestamp_pb2
import nebius.aio.client as client
import nebius.aio.operation as operation
import nebius.aio.request as request_1
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.common.v1alpha1 as v1alpha1_1
import nebius.api.nebius.common.v1alpha1.operation_pb2 as operation_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.backup_pb2 as backup_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.backup_service_pb2 as backup_service_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.cluster_pb2 as cluster_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.cluster_service_pb2 as cluster_service_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.config as config_1
import nebius.api.nebius.msp.postgresql.v1alpha1.config.postgresql_pb2 as postgresql_pb2
import nebius.api.nebius.msp.postgresql.v1alpha1.template_pb2 as template_pb2
import nebius.api.nebius.msp.v1alpha1 as v1alpha1_2
import nebius.api.nebius.msp.v1alpha1.cluster_pb2 as cluster_pb2_1
import nebius.api.nebius.msp.v1alpha1.resource as resource_1
import nebius.api.nebius.msp.v1alpha1.resource.template_pb2 as template_pb2_1
import nebius.base.fieldmask_protobuf as fieldmask_protobuf
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import nebius.base.protos.well_known as well_known_1
import typing as typing
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/msp/postgresql/v1alpha1/backup.proto
class Backup(pb_classes.Message):
    __PB2_CLASS__ = backup_pb2.Backup
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.Backup",backup_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "creation_start": well_known_1.ts_mask,
        "creation_finish": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        source_cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        creation_start: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        creation_finish: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        source_cluster_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        source_cluster_visible: "builtins.bool|None|unset.UnsetType" = unset.Unset,
        on_demand: "builtins.bool|None|unset.UnsetType" = unset.Unset,
        size_bytes: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(source_cluster_id, unset.UnsetType):
            self.source_cluster_id = source_cluster_id
        if not isinstance(creation_start, unset.UnsetType):
            self.creation_start = creation_start
        if not isinstance(creation_finish, unset.UnsetType):
            self.creation_finish = creation_finish
        if not isinstance(source_cluster_name, unset.UnsetType):
            self.source_cluster_name = source_cluster_name
        if not isinstance(source_cluster_visible, unset.UnsetType):
            self.source_cluster_visible = source_cluster_visible
        if not isinstance(on_demand, unset.UnsetType):
            self.on_demand = on_demand
        if not isinstance(size_bytes, unset.UnsetType):
            self.size_bytes = size_bytes
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "source_cluster_id",
            "creation_start",
            "creation_finish",
            "source_cluster_name",
            "source_cluster_visible",
            "on_demand",
            "size_bytes",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Required. ID of the backup.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def source_cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster that the backup was created for.
        """
        
        return super()._get_field("source_cluster_id", explicit_presence=False,
        )
    @source_cluster_id.setter
    def source_cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("source_cluster_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def creation_start(self) -> "datetime.datetime":
        """
        Creation start timestamp.
        """
        
        return super()._get_field("creation_start", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @creation_start.setter
    def creation_start(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("creation_start",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def creation_finish(self) -> "datetime.datetime":
        """
        Creation finish timestamp.
        """
        
        return super()._get_field("creation_finish", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @creation_finish.setter
    def creation_finish(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("creation_finish",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def source_cluster_name(self) -> "builtins.str":
        """
        Name of the PostgreSQL cluster that the backup was created for.
        """
        
        return super()._get_field("source_cluster_name", explicit_presence=False,
        )
    @source_cluster_name.setter
    def source_cluster_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("source_cluster_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def source_cluster_visible(self) -> "builtins.bool":
        """
        Is PostgreSQL cluster that the backup was created for visible.
        """
        
        return super()._get_field("source_cluster_visible", explicit_presence=False,
        )
    @source_cluster_visible.setter
    def source_cluster_visible(self, value: "builtins.bool|None") -> None:
        return super()._set_field("source_cluster_visible",value,explicit_presence=False,
        )
    
    @builtins.property
    def on_demand(self) -> "builtins.bool":
        """
        Is the backup an on-demand manual backup.
        """
        
        return super()._get_field("on_demand", explicit_presence=False,
        )
    @on_demand.setter
    def on_demand(self, value: "builtins.bool|None") -> None:
        return super()._set_field("on_demand",value,explicit_presence=False,
        )
    
    @builtins.property
    def size_bytes(self) -> "builtins.int":
        """
        Size of the backup in bytes.
        """
        
        return super()._get_field("size_bytes", explicit_presence=False,
        )
    @size_bytes.setter
    def size_bytes(self, value: "builtins.int|None") -> None:
        return super()._set_field("size_bytes",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "source_cluster_id":"source_cluster_id",
        "creation_start":"creation_start",
        "creation_finish":"creation_finish",
        "source_cluster_name":"source_cluster_name",
        "source_cluster_visible":"source_cluster_visible",
        "on_demand":"on_demand",
        "size_bytes":"size_bytes",
    }
    
# file: nebius/msp/postgresql/v1alpha1/backup_service.proto
class GetBackupRequest(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.GetBackupRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.GetBackupRequest",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        backup_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
        if not isinstance(backup_id, unset.UnsetType):
            self.backup_id = backup_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cluster_id",
            "backup_id",
        ]
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster that holds backup.
        """
        
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def backup_id(self) -> "builtins.str":
        """
        ID of the designated backup.
        """
        
        return super()._get_field("backup_id", explicit_presence=False,
        )
    @backup_id.setter
    def backup_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("backup_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cluster_id":"cluster_id",
        "backup_id":"backup_id",
    }
    
class ListBackupsRequest(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.ListBackupsRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ListBackupsRequest",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Identifier of IAM container to list backups from.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
    }
    
class ListBackupsByClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.ListBackupsByClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ListBackupsByClusterRequest",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cluster_id",
        ]
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster to list backups from.
        """
        
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cluster_id":"cluster_id",
    }
    
class ListBackupsResponse(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.ListBackupsResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ListBackupsResponse",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        backups: "abc.Iterable[Backup]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(backups, unset.UnsetType):
            self.backups = backups
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "backups",
        ]
    
    @builtins.property
    def backups(self) -> "abc.MutableSequence[Backup]":
        """
        List of backups.
        """
        
        return super()._get_field("backups", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Backup,None,None),
        )
    @backups.setter
    def backups(self, value: "abc.Iterable[Backup]|None") -> None:
        return super()._set_field("backups",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "backups":"backups",
    }
    
class CreateBackupRequest(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.CreateBackupRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.CreateBackupRequest",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cluster_id",
        ]
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster that holds backup.
        """
        
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cluster_id":"cluster_id",
    }
    
class DeleteBackupRequest(pb_classes.Message):
    __PB2_CLASS__ = backup_service_pb2.DeleteBackupRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.DeleteBackupRequest",backup_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        backup_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
        if not isinstance(backup_id, unset.UnsetType):
            self.backup_id = backup_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cluster_id",
            "backup_id",
        ]
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster that holds backup.
        """
        
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def backup_id(self) -> "builtins.str":
        """
        ID of the designated backup.
        """
        
        return super()._get_field("backup_id", explicit_presence=False,
        )
    @backup_id.setter
    def backup_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("backup_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cluster_id":"cluster_id",
        "backup_id":"backup_id",
    }
    

class BackupServiceClient(client.ClientWithOperations[v1alpha1_1.Operation,v1alpha1_1.OperationServiceClient]):
    """
    A set of methods for managing PostgreSQL Cluster backups.
    
    This class provides the client methods for the ``.nebius.msp.postgresql.v1alpha1.BackupService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.msp.postgresql.v1alpha1.BackupService",backup_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.msp.postgresql.v1alpha1.BackupService"
    __operation_type__ = v1alpha1_1.Operation
    __operation_service_class__ = v1alpha1_1.OperationServiceClient
    __operation_source_method__ = "Create"
    """The method name that can be used to fetch the address channel for the operation."""
    
    def get(self,
        request: "GetBackupRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetBackupRequest","Backup"]:
        """
        Returns the specified PostgreSQL Cluster backup.
        To get the list of available PostgreSQL Cluster backups, make a [List] or [ListByCluster] request.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.GetBackupRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.Backup`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=backup_pb2.Backup,
            result_wrapper=pb_classes.simple_wrapper(Backup),
            **kwargs,
        )
    
    def list(self,
        request: "ListBackupsRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListBackupsRequest","ListBackupsResponse"]:
        """
        Retrieves the list of PostgreSQL Cluster backups by project.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListBackupsRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListBackupsResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=backup_service_pb2.ListBackupsResponse,
            result_wrapper=pb_classes.simple_wrapper(ListBackupsResponse),
            **kwargs,
        )
    
    def list_by_cluster(self,
        request: "ListBackupsByClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListBackupsByClusterRequest","ListBackupsResponse"]:
        """
        Retrieves the list of PostgreSQL Cluster backups by cluster.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListBackupsByClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListBackupsResponse`.
        """
        
        return super().request(
            method="ListByCluster",
            request=request,
            result_pb2_class=backup_service_pb2.ListBackupsResponse,
            result_wrapper=pb_classes.simple_wrapper(ListBackupsResponse),
            **kwargs,
        )
    
    def create(self,
        request: "CreateBackupRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["CreateBackupRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Creates a new on-demand backup.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.CreateBackupRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Create",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def delete(self,
        request: "DeleteBackupRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["DeleteBackupRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Deletes an on-demand backup.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.DeleteBackupRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Delete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    

# file: nebius/msp/postgresql/v1alpha1/template.proto
class TemplateSpec(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.TemplateSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.TemplateSpec",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        resources: "resource_1.ResourcesSpec|template_pb2_1.ResourcesSpec|None|unset.UnsetType" = unset.Unset,
        hosts: "resource_1.HostSpec|template_pb2_1.HostSpec|None|unset.UnsetType" = unset.Unset,
        disk: "resource_1.DiskSpec|template_pb2_1.DiskSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(resources, unset.UnsetType):
            self.resources = resources
        if not isinstance(hosts, unset.UnsetType):
            self.hosts = hosts
        if not isinstance(disk, unset.UnsetType):
            self.disk = disk
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "resources",
            "hosts",
            "disk",
        ]
    
    @builtins.property
    def resources(self) -> "resource_1.ResourcesSpec":
        """
        reduced msp.v1alpha1.resource.TemplateSpec
        """
        
        return super()._get_field("resources", explicit_presence=False,
        wrap=resource_1.ResourcesSpec,
        )
    @resources.setter
    def resources(self, value: "resource_1.ResourcesSpec|template_pb2_1.ResourcesSpec|None") -> None:
        return super()._set_field("resources",value,explicit_presence=False,
        )
    
    @builtins.property
    def hosts(self) -> "resource_1.HostSpec":
        return super()._get_field("hosts", explicit_presence=False,
        wrap=resource_1.HostSpec,
        )
    @hosts.setter
    def hosts(self, value: "resource_1.HostSpec|template_pb2_1.HostSpec|None") -> None:
        return super()._set_field("hosts",value,explicit_presence=False,
        )
    
    @builtins.property
    def disk(self) -> "resource_1.DiskSpec":
        return super()._get_field("disk", explicit_presence=False,
        wrap=resource_1.DiskSpec,
        )
    @disk.setter
    def disk(self, value: "resource_1.DiskSpec|template_pb2_1.DiskSpec|None") -> None:
        return super()._set_field("disk",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "resources":"resources",
        "hosts":"hosts",
        "disk":"disk",
    }
    
# file: nebius/msp/postgresql/v1alpha1/cluster.proto
class Cluster(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.Cluster
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.Cluster",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "ClusterSpec|cluster_pb2.ClusterSpec|None|unset.UnsetType" = unset.Unset,
        status: "ClusterStatus|cluster_pb2.ClusterStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "ClusterSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=ClusterSpec,
        )
    @spec.setter
    def spec(self, value: "ClusterSpec|cluster_pb2.ClusterSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "ClusterStatus":
        return super()._get_field("status", explicit_presence=False,
        wrap=ClusterStatus,
        )
    @status.setter
    def status(self, value: "ClusterStatus|cluster_pb2.ClusterStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class ConnectionPoolerConfig(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.ConnectionPoolerConfig
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ConnectionPoolerConfig",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class PoolingMode(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.msp.postgresql.v1alpha1.ConnectionPoolerConfig.PoolingMode",cluster_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        POOLING_MODE_UNSPECIFIED = 0
        SESSION = 1
        """
        Session pooling mode.
        """
        
        TRANSACTION = 2
        """
        Transaction pooling mode.
        """
        
    
    class __OneOfClass__max_pool_size__(pb_classes.OneOf):
        name: builtins.str= "_max_pool_size"
        
        def __init__(self, msg: "ConnectionPoolerConfig") -> None:
            super().__init__()
            self._message: "ConnectionPoolerConfig" = msg
    
    class __OneOfClass__max_pool_size_max_pool_size__(__OneOfClass__max_pool_size__):
        field: typing.Literal["max_pool_size"] = "max_pool_size"
        
        def __init__(self, msg: "ConnectionPoolerConfig") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.int":
            return self._message.max_pool_size
    
    @builtins.property
    def _max_pool_size(self) -> __OneOfClass__max_pool_size_max_pool_size__|None:
        field_name_1: str|None = super().which_field_in_oneof("_max_pool_size")
        match field_name_1:
            case "max_pool_size":
                return self.__OneOfClass__max_pool_size_max_pool_size__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        pooling_mode: "ConnectionPoolerConfig.PoolingMode|cluster_pb2.ConnectionPoolerConfig.PoolingMode|None|unset.UnsetType" = unset.Unset,
        max_pool_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(pooling_mode, unset.UnsetType):
            self.pooling_mode = pooling_mode
        if not isinstance(max_pool_size, unset.UnsetType):
            self.max_pool_size = max_pool_size
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "pooling_mode",
            "max_pool_size",
            "_max_pool_size",
            "PoolingMode",
        ]
    
    @builtins.property
    def pooling_mode(self) -> "ConnectionPoolerConfig.PoolingMode":
        """
        Mode that the connection pooler is working in.
        """
        
        return super()._get_field("pooling_mode", explicit_presence=False,
        wrap=ConnectionPoolerConfig.PoolingMode,
        )
    @pooling_mode.setter
    def pooling_mode(self, value: "ConnectionPoolerConfig.PoolingMode|cluster_pb2.ConnectionPoolerConfig.PoolingMode|None") -> None:
        return super()._set_field("pooling_mode",value,explicit_presence=False,
        )
    
    @builtins.property
    def max_pool_size(self) -> "builtins.int|None":
        """
        Maximum number of connections in the pool for a single user/database pair.
        """
        
        return super()._get_field("max_pool_size", explicit_presence=True,
        )
    @max_pool_size.setter
    def max_pool_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("max_pool_size",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "pooling_mode":"pooling_mode",
        "max_pool_size":"max_pool_size",
        "_max_pool_size":"_max_pool_size",
        "PoolingMode":"PoolingMode",
    }
    
class ClusterSpec(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.ClusterSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ClusterSpec",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        network_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        config: "ConfigSpec|cluster_pb2.ConfigSpec|None|unset.UnsetType" = unset.Unset,
        bootstrap: "BootstrapSpec|cluster_pb2.BootstrapSpec|None|unset.UnsetType" = unset.Unset,
        backup: "BackupSpec|cluster_pb2.BackupSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(description, unset.UnsetType):
            self.description = description
        if not isinstance(network_id, unset.UnsetType):
            self.network_id = network_id
        if not isinstance(config, unset.UnsetType):
            self.config = config
        if not isinstance(bootstrap, unset.UnsetType):
            self.bootstrap = bootstrap
        if not isinstance(backup, unset.UnsetType):
            self.backup = backup
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "description",
            "network_id",
            "config",
            "bootstrap",
            "backup",
        ]
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        Description of the PostgreSQL cluster.
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    @builtins.property
    def network_id(self) -> "builtins.str":
        """
        Network ID in which the cluster is created.
        """
        
        return super()._get_field("network_id", explicit_presence=False,
        )
    @network_id.setter
    def network_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("network_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def config(self) -> "ConfigSpec":
        return super()._get_field("config", explicit_presence=False,
        wrap=ConfigSpec,
        )
    @config.setter
    def config(self, value: "ConfigSpec|cluster_pb2.ConfigSpec|None") -> None:
        return super()._set_field("config",value,explicit_presence=False,
        )
    
    @builtins.property
    def bootstrap(self) -> "BootstrapSpec":
        return super()._get_field("bootstrap", explicit_presence=False,
        wrap=BootstrapSpec,
        )
    @bootstrap.setter
    def bootstrap(self, value: "BootstrapSpec|cluster_pb2.BootstrapSpec|None") -> None:
        return super()._set_field("bootstrap",value,explicit_presence=False,
        )
    
    @builtins.property
    def backup(self) -> "BackupSpec":
        return super()._get_field("backup", explicit_presence=False,
        wrap=BackupSpec,
        )
    @backup.setter
    def backup(self, value: "BackupSpec|cluster_pb2.BackupSpec|None") -> None:
        return super()._set_field("backup",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "description":"description",
        "network_id":"network_id",
        "config":"config",
        "bootstrap":"bootstrap",
        "backup":"backup",
    }
    
class ClusterStatus(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.ClusterStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ClusterStatus",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        phase: "v1alpha1_2.ClusterStatus.Phase|cluster_pb2_1.ClusterStatus.Phase|None|unset.UnsetType" = unset.Unset,
        state: "v1alpha1_2.ClusterStatus.State|cluster_pb2_1.ClusterStatus.State|None|unset.UnsetType" = unset.Unset,
        preset_details: "resource_1.PresetDetails|template_pb2_1.PresetDetails|None|unset.UnsetType" = unset.Unset,
        connection_endpoints: "Endpoints|cluster_pb2.Endpoints|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(phase, unset.UnsetType):
            self.phase = phase
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(preset_details, unset.UnsetType):
            self.preset_details = preset_details
        if not isinstance(connection_endpoints, unset.UnsetType):
            self.connection_endpoints = connection_endpoints
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "phase",
            "state",
            "preset_details",
            "connection_endpoints",
        ]
    
    @builtins.property
    def phase(self) -> "v1alpha1_2.ClusterStatus.Phase":
        """
        Current phase of the cluster.
        """
        
        return super()._get_field("phase", explicit_presence=False,
        wrap=v1alpha1_2.ClusterStatus.Phase,
        )
    @phase.setter
    def phase(self, value: "v1alpha1_2.ClusterStatus.Phase|cluster_pb2_1.ClusterStatus.Phase|None") -> None:
        return super()._set_field("phase",value,explicit_presence=False,
        )
    
    @builtins.property
    def state(self) -> "v1alpha1_2.ClusterStatus.State":
        """
        State reflects substatus of the phase to define whether it's healthy or not.
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=v1alpha1_2.ClusterStatus.State,
        )
    @state.setter
    def state(self, value: "v1alpha1_2.ClusterStatus.State|cluster_pb2_1.ClusterStatus.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def preset_details(self) -> "resource_1.PresetDetails":
        """
        Cluster resource preset details
        """
        
        return super()._get_field("preset_details", explicit_presence=False,
        wrap=resource_1.PresetDetails,
        )
    @preset_details.setter
    def preset_details(self, value: "resource_1.PresetDetails|template_pb2_1.PresetDetails|None") -> None:
        return super()._set_field("preset_details",value,explicit_presence=False,
        )
    
    @builtins.property
    def connection_endpoints(self) -> "Endpoints":
        """
        Connection endpoints of the PostgreSQL cluster.
        """
        
        return super()._get_field("connection_endpoints", explicit_presence=False,
        wrap=Endpoints,
        )
    @connection_endpoints.setter
    def connection_endpoints(self, value: "Endpoints|cluster_pb2.Endpoints|None") -> None:
        return super()._set_field("connection_endpoints",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "phase":"phase",
        "state":"state",
        "preset_details":"preset_details",
        "connection_endpoints":"connection_endpoints",
    }
    
class Endpoints(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.Endpoints
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.Endpoints",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        private_read_write: "builtins.str|None|unset.UnsetType" = unset.Unset,
        private_read_only: "builtins.str|None|unset.UnsetType" = unset.Unset,
        public_read_write: "builtins.str|None|unset.UnsetType" = unset.Unset,
        public_read_only: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(private_read_write, unset.UnsetType):
            self.private_read_write = private_read_write
        if not isinstance(private_read_only, unset.UnsetType):
            self.private_read_only = private_read_only
        if not isinstance(public_read_write, unset.UnsetType):
            self.public_read_write = public_read_write
        if not isinstance(public_read_only, unset.UnsetType):
            self.public_read_only = public_read_only
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "private_read_write",
            "private_read_only",
            "public_read_write",
            "public_read_only",
        ]
    
    @builtins.property
    def private_read_write(self) -> "builtins.str":
        """
        Read write endpoint of the PostgreSQL cluster accessible from the private VPC.
        """
        
        return super()._get_field("private_read_write", explicit_presence=False,
        )
    @private_read_write.setter
    def private_read_write(self, value: "builtins.str|None") -> None:
        return super()._set_field("private_read_write",value,explicit_presence=False,
        )
    
    @builtins.property
    def private_read_only(self) -> "builtins.str":
        """
        Read only endpoint of the PostgreSQL cluster accessible from the private VPC.
        """
        
        return super()._get_field("private_read_only", explicit_presence=False,
        )
    @private_read_only.setter
    def private_read_only(self, value: "builtins.str|None") -> None:
        return super()._set_field("private_read_only",value,explicit_presence=False,
        )
    
    @builtins.property
    def public_read_write(self) -> "builtins.str":
        """
        Read write endpoint of the PostgreSQL cluster accessible from the internet.
        """
        
        return super()._get_field("public_read_write", explicit_presence=False,
        )
    @public_read_write.setter
    def public_read_write(self, value: "builtins.str|None") -> None:
        return super()._set_field("public_read_write",value,explicit_presence=False,
        )
    
    @builtins.property
    def public_read_only(self) -> "builtins.str":
        """
        Read only endpoint of the PostgreSQL cluster accessible from the internet.
        """
        
        return super()._get_field("public_read_only", explicit_presence=False,
        )
    @public_read_only.setter
    def public_read_only(self, value: "builtins.str|None") -> None:
        return super()._set_field("public_read_only",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "private_read_write":"private_read_write",
        "private_read_only":"private_read_only",
        "public_read_write":"public_read_write",
        "public_read_only":"public_read_only",
    }
    
class ConfigSpec(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.ConfigSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ConfigSpec",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass_config__(pb_classes.OneOf):
        name: builtins.str= "config"
        
        def __init__(self, msg: "ConfigSpec") -> None:
            super().__init__()
            self._message: "ConfigSpec" = msg
    
    class __OneOfClass_config_postgresql_config_16__(__OneOfClass_config__):
        field: typing.Literal["postgresql_config_16"] = "postgresql_config_16"
        
        def __init__(self, msg: "ConfigSpec") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "config_1.PostgresqlConfig16":
            return self._message.postgresql_config_16
    
    @builtins.property
    def config(self) -> __OneOfClass_config_postgresql_config_16__|None:
        """
        Config of the PostgreSQL cluster.
        """
        
        field_name_1: str|None = super().which_field_in_oneof("config")
        match field_name_1:
            case "postgresql_config_16":
                return self.__OneOfClass_config_postgresql_config_16__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        version: "builtins.str|None|unset.UnsetType" = unset.Unset,
        pooler_config: "ConnectionPoolerConfig|cluster_pb2.ConnectionPoolerConfig|None|unset.UnsetType" = unset.Unset,
        postgresql_config_16: "config_1.PostgresqlConfig16|postgresql_pb2.PostgresqlConfig16|None|unset.UnsetType" = unset.Unset,
        public_access: "builtins.bool|None|unset.UnsetType" = unset.Unset,
        template: "TemplateSpec|template_pb2.TemplateSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(version, unset.UnsetType):
            self.version = version
        if not isinstance(pooler_config, unset.UnsetType):
            self.pooler_config = pooler_config
        if not isinstance(postgresql_config_16, unset.UnsetType):
            self.postgresql_config_16 = postgresql_config_16
        if not isinstance(public_access, unset.UnsetType):
            self.public_access = public_access
        if not isinstance(template, unset.UnsetType):
            self.template = template
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "version",
            "pooler_config",
            "postgresql_config_16",
            "public_access",
            "template",
            "config",
        ]
    
    @builtins.property
    def version(self) -> "builtins.str":
        """
        Version of PostgreSQL used in the cluster.
        Possible values: ``16``
        """
        
        return super()._get_field("version", explicit_presence=False,
        )
    @version.setter
    def version(self, value: "builtins.str|None") -> None:
        return super()._set_field("version",value,explicit_presence=False,
        )
    
    @builtins.property
    def pooler_config(self) -> "ConnectionPoolerConfig":
        """
        Configuration of the connection pooler.
        """
        
        return super()._get_field("pooler_config", explicit_presence=False,
        wrap=ConnectionPoolerConfig,
        )
    @pooler_config.setter
    def pooler_config(self, value: "ConnectionPoolerConfig|cluster_pb2.ConnectionPoolerConfig|None") -> None:
        return super()._set_field("pooler_config",value,explicit_presence=False,
        )
    
    @builtins.property
    def postgresql_config_16(self) -> "config_1.PostgresqlConfig16|None":
        """
        Configuration parameters for postgres
        """
        
        return super()._get_field("postgresql_config_16", explicit_presence=True,
        wrap=config_1.PostgresqlConfig16,
        )
    @postgresql_config_16.setter
    def postgresql_config_16(self, value: "config_1.PostgresqlConfig16|postgresql_pb2.PostgresqlConfig16|None") -> None:
        return super()._set_field("postgresql_config_16",value,explicit_presence=True,
        )
    
    @builtins.property
    def public_access(self) -> "builtins.bool":
        """
        Either make cluster public accessible or accessible only via private VPC.
        """
        
        return super()._get_field("public_access", explicit_presence=False,
        )
    @public_access.setter
    def public_access(self, value: "builtins.bool|None") -> None:
        return super()._set_field("public_access",value,explicit_presence=False,
        )
    
    @builtins.property
    def template(self) -> "TemplateSpec":
        """
        Resources allocated to PostgreSQL hosts.
        """
        
        return super()._get_field("template", explicit_presence=False,
        wrap=TemplateSpec,
        )
    @template.setter
    def template(self, value: "TemplateSpec|template_pb2.TemplateSpec|None") -> None:
        return super()._set_field("template",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "version":"version",
        "pooler_config":"pooler_config",
        "postgresql_config_16":"postgresql_config_16",
        "public_access":"public_access",
        "template":"template",
        "config":"config",
    }
    
class BootstrapSpec(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.BootstrapSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.BootstrapSpec",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        user_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        user_password: "builtins.str|None|unset.UnsetType" = unset.Unset,
        db_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(user_name, unset.UnsetType):
            self.user_name = user_name
        if not isinstance(user_password, unset.UnsetType):
            self.user_password = user_password
        if not isinstance(db_name, unset.UnsetType):
            self.db_name = db_name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "user_name",
            "user_password",
            "db_name",
        ]
    
    @builtins.property
    def user_name(self) -> "builtins.str":
        """
        Name of the bootstrap PostgreSQL user.
        """
        
        return super()._get_field("user_name", explicit_presence=False,
        )
    @user_name.setter
    def user_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("user_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def user_password(self) -> "builtins.str":
        """
        Password of the bootstrap PostgreSQL user.
        """
        
        return super()._get_field("user_password", explicit_presence=False,
        )
    @user_password.setter
    def user_password(self, value: "builtins.str|None") -> None:
        return super()._set_field("user_password",value,explicit_presence=False,
        )
    
    @builtins.property
    def db_name(self) -> "builtins.str":
        """
        Name of the PostgreSQL database. 1-63 characters long.
        """
        
        return super()._get_field("db_name", explicit_presence=False,
        )
    @db_name.setter
    def db_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("db_name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "user_name":"user_name",
        "user_password":"user_password",
        "db_name":"db_name",
    }
    
class BackupSpec(pb_classes.Message):
    __PB2_CLASS__ = cluster_pb2.BackupSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.BackupSpec",cluster_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        backup_window_start: "builtins.str|None|unset.UnsetType" = unset.Unset,
        retention_policy: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(backup_window_start, unset.UnsetType):
            self.backup_window_start = backup_window_start
        if not isinstance(retention_policy, unset.UnsetType):
            self.retention_policy = retention_policy
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "backup_window_start",
            "retention_policy",
        ]
    
    @builtins.property
    def backup_window_start(self) -> "builtins.str":
        """
        Backup window start in "HH:MM:SS" format (UTC Time)
        """
        
        return super()._get_field("backup_window_start", explicit_presence=False,
        )
    @backup_window_start.setter
    def backup_window_start(self, value: "builtins.str|None") -> None:
        return super()._set_field("backup_window_start",value,explicit_presence=False,
        )
    
    @builtins.property
    def retention_policy(self) -> "builtins.str":
        """
        Retention policy to be used for backups and WALs (i.e. '7d')
        """
        
        return super()._get_field("retention_policy", explicit_presence=False,
        )
    @retention_policy.setter
    def retention_policy(self, value: "builtins.str|None") -> None:
        return super()._set_field("retention_policy",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "backup_window_start":"backup_window_start",
        "retention_policy":"retention_policy",
    }
    
# file: nebius/msp/postgresql/v1alpha1/cluster_service.proto
class GetClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.GetClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.GetClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the PostgreSQL Cluster resource to return.
        To get the cluster ID use a [ClusterService.List] request.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class GetClusterForBackupRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.GetClusterForBackupRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.GetClusterForBackupRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        backup_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(backup_id, unset.UnsetType):
            self.backup_id = backup_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "backup_id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster that holds backup.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def backup_id(self) -> "builtins.str":
        """
        ID of the backup for which source cluster is requested.
        """
        
        return super()._get_field("backup_id", explicit_presence=False,
        )
    @backup_id.setter
    def backup_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("backup_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "backup_id":"backup_id",
    }
    
class ListClustersRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.ListClustersRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ListClustersRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Identifier of IAM container to list clusters from.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        Specifies the maximum number of items to return in the response.
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        Token for pagination, allowing the retrieval of the next set of results.
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
    }
    
class ListClustersResponse(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.ListClustersResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.ListClustersResponse",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        clusters: "abc.Iterable[Cluster]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(clusters, unset.UnsetType):
            self.clusters = clusters
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "clusters",
            "next_page_token",
        ]
    
    @builtins.property
    def clusters(self) -> "abc.MutableSequence[Cluster]":
        """
        List of PostgreSQL Cluster resources.
        """
        
        return super()._get_field("clusters", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Cluster,None,None),
        )
    @clusters.setter
    def clusters(self, value: "abc.Iterable[Cluster]|None") -> None:
        return super()._set_field("clusters",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        This token allows you to get the next page of results for list requests. If the number of results
        is larger than [ListClustersRequest.page_size], use the [next_page_token] as the value
        for the [ListClustersRequest.page_token] parameter in the next list request. Each subsequent
        list request will have its own [next_page_token] to continue paging through the results.
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "clusters":"clusters",
        "next_page_token":"next_page_token",
    }
    
class CreateClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.CreateClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.CreateClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "ClusterSpec|cluster_pb2.ClusterSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        Metadata associated with the new cluster.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "ClusterSpec":
        """
        Specification for the new cluster.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=ClusterSpec,
        )
    @spec.setter
    def spec(self, value: "ClusterSpec|cluster_pb2.ClusterSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class RestoreClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.RestoreClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.RestoreClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "recovery_time": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "ClusterSpec|cluster_pb2.ClusterSpec|None|unset.UnsetType" = unset.Unset,
        backup_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        source_cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        recovery_time: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(backup_id, unset.UnsetType):
            self.backup_id = backup_id
        if not isinstance(source_cluster_id, unset.UnsetType):
            self.source_cluster_id = source_cluster_id
        if not isinstance(recovery_time, unset.UnsetType):
            self.recovery_time = recovery_time
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "backup_id",
            "source_cluster_id",
            "recovery_time",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        Metadata associated with the new cluster.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "ClusterSpec":
        """
        Specification for the new cluster.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=ClusterSpec,
        )
    @spec.setter
    def spec(self, value: "ClusterSpec|cluster_pb2.ClusterSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def backup_id(self) -> "builtins.str":
        """
        ID of the backup to restore from.
        """
        
        return super()._get_field("backup_id", explicit_presence=False,
        )
    @backup_id.setter
    def backup_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("backup_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def source_cluster_id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster to restore from.
        """
        
        return super()._get_field("source_cluster_id", explicit_presence=False,
        )
    @source_cluster_id.setter
    def source_cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("source_cluster_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def recovery_time(self) -> "datetime.datetime":
        """
        Timestamp for point in time recovery.
        """
        
        return super()._get_field("recovery_time", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @recovery_time.setter
    def recovery_time(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("recovery_time",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "backup_id":"backup_id",
        "source_cluster_id":"source_cluster_id",
        "recovery_time":"recovery_time",
    }
    
class DeleteClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.DeleteClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.DeleteClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the PostgreSQL cluster to delete.
        To get the PostgreSQL cluster ID, use a [ClusterService.List] request.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class UpdateClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.UpdateClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.UpdateClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "ClusterSpec|cluster_pb2.ClusterSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        Metadata associated with the cluster. Must include id of the cluster we are going to update.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "ClusterSpec":
        """
        Updated specification for the cluster.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=ClusterSpec,
        )
    @spec.setter
    def spec(self, value: "ClusterSpec|cluster_pb2.ClusterSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class StopClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.StopClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.StopClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the PostgreSQL Cluster resource to pause.
        To get the cluster ID use a [ClusterService.List] request.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class StartClusterRequest(pb_classes.Message):
    __PB2_CLASS__ = cluster_service_pb2.StartClusterRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.postgresql.v1alpha1.StartClusterRequest",cluster_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the PostgreSQL Cluster resource to resume.
        To get the cluster ID use a [ClusterService.List] request.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    

class ClusterServiceClient(client.ClientWithOperations[v1alpha1_1.Operation,v1alpha1_1.OperationServiceClient]):
    """
    A set of methods for managing PostgreSQL Cluster resources.
    
    This class provides the client methods for the ``.nebius.msp.postgresql.v1alpha1.ClusterService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.msp.postgresql.v1alpha1.ClusterService",cluster_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.msp.postgresql.v1alpha1.ClusterService"
    __operation_type__ = v1alpha1_1.Operation
    __operation_service_class__ = v1alpha1_1.OperationServiceClient
    __operation_source_method__ = "Create"
    """The method name that can be used to fetch the address channel for the operation."""
    
    def get(self,
        request: "GetClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetClusterRequest","Cluster"]:
        """
        Returns the specified PostgreSQL Cluster resource.
        To get the list of available PostgreSQL Cluster resources, make a [List] request.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.GetClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.Cluster`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=cluster_pb2.Cluster,
            result_wrapper=pb_classes.simple_wrapper(Cluster),
            **kwargs,
        )
    
    def get_by_name(self,
        request: "v1_1.GetByNameRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["v1_1.GetByNameRequest","Cluster"]:
        """
        Returns the specified PostgreSQL Cluster resource by name.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.common.v1.GetByNameRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.Cluster`.
        """
        
        return super().request(
            method="GetByName",
            request=request,
            result_pb2_class=cluster_pb2.Cluster,
            result_wrapper=pb_classes.simple_wrapper(Cluster),
            **kwargs,
        )
    
    def get_for_backup(self,
        request: "GetClusterForBackupRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetClusterForBackupRequest","Cluster"]:
        """
        Returns the specified PostgreSQL Cluster resource for backup.
        It should be used as a hint of cluster configuration in case of backup restoration.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.GetClusterForBackupRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.Cluster`.
        """
        
        return super().request(
            method="GetForBackup",
            request=request,
            result_pb2_class=cluster_pb2.Cluster,
            result_wrapper=pb_classes.simple_wrapper(Cluster),
            **kwargs,
        )
    
    def list(self,
        request: "ListClustersRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListClustersRequest","ListClustersResponse"]:
        """
        Retrieves the list of PostgreSQL Cluster resources that belong
        to the specified folder.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListClustersRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.msp.postgresql.v1alpha1.ListClustersResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=cluster_service_pb2.ListClustersResponse,
            result_wrapper=pb_classes.simple_wrapper(ListClustersResponse),
            **kwargs,
        )
    
    def create(self,
        request: "CreateClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["CreateClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Creates a PostgreSQL cluster in the specified folder.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.CreateClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Create",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def delete(self,
        request: "DeleteClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["DeleteClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Deletes the specified PostgreSQL cluster.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.DeleteClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Delete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def update(self,
        request: "UpdateClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["UpdateClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Updates the PostgreSQL cluster.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.UpdateClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        kwargs['metadata'] = fieldmask_protobuf.ensure_reset_mask_in_metadata(request, kwargs.get('metadata', None))
        return super().request(
            method="Update",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def restore(self,
        request: "RestoreClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["RestoreClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Creates a new PostgreSQL cluster from a previously created backup.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.RestoreClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Restore",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def stop(self,
        request: "StopClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["StopClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Suspends the PostgreSQL cluster to save resources.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.StopClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Stop",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def start(self,
        request: "StartClusterRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["StartClusterRequest","operation.Operation[v1alpha1_1.Operation]"]:
        """
        Wakes up suspended PostgreSQL cluster.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.msp.postgresql.v1alpha1.StartClusterRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1alpha1.Operation`.
        """
        
        return super().request(
            method="Start",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "Backup",
    "GetBackupRequest",
    "ListBackupsRequest",
    "ListBackupsByClusterRequest",
    "ListBackupsResponse",
    "CreateBackupRequest",
    "DeleteBackupRequest",
    "BackupServiceClient",
    "TemplateSpec",
    "Cluster",
    "ConnectionPoolerConfig",
    "ClusterSpec",
    "ClusterStatus",
    "Endpoints",
    "ConfigSpec",
    "BootstrapSpec",
    "BackupSpec",
    "GetClusterRequest",
    "GetClusterForBackupRequest",
    "ListClustersRequest",
    "ListClustersResponse",
    "CreateClusterRequest",
    "RestoreClusterRequest",
    "DeleteClusterRequest",
    "UpdateClusterRequest",
    "StopClusterRequest",
    "StartClusterRequest",
    "ClusterServiceClient",
]



from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Request(_message.Message):
    __slots__ = ["client_ip", "user_agent", "request_id", "parameters", "idempotency_id", "trace_id", "ja3_fingerprint"]
    CLIENT_IP_FIELD_NUMBER: _ClassVar[int]
    USER_AGENT_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_ID_FIELD_NUMBER: _ClassVar[int]
    TRACE_ID_FIELD_NUMBER: _ClassVar[int]
    JA3_FINGERPRINT_FIELD_NUMBER: _ClassVar[int]
    client_ip: str
    user_agent: str
    request_id: str
    parameters: _struct_pb2.Struct
    idempotency_id: str
    trace_id: str
    ja3_fingerprint: str
    def __init__(self, client_ip: _Optional[str] = ..., user_agent: _Optional[str] = ..., request_id: _Optional[str] = ..., parameters: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., idempotency_id: _Optional[str] = ..., trace_id: _Optional[str] = ..., ja3_fingerprint: _Optional[str] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.iam.v1 import service_account_pb2 as _service_account_pb2
from nebius.api.nebius.iam.v1 import tenant_user_account_pb2 as _tenant_user_account_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GroupMembership(_message.Message):
    __slots__ = ["metadata", "spec", "status", "revoke_at"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    REVOKE_AT_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: GroupMembershipSpec
    status: GroupMembershipStatus
    revoke_at: _timestamp_pb2.Timestamp
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[GroupMembershipSpec, _Mapping]] = ..., status: _Optional[_Union[GroupMembershipStatus, _Mapping]] = ..., revoke_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class GroupMembershipSpec(_message.Message):
    __slots__ = ["member_id"]
    MEMBER_ID_FIELD_NUMBER: _ClassVar[int]
    member_id: str
    def __init__(self, member_id: _Optional[str] = ...) -> None: ...

class GroupMembershipStatus(_message.Message):
    __slots__ = ["tenant_user_account_status", "service_account_status"]
    TENANT_USER_ACCOUNT_STATUS_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_STATUS_FIELD_NUMBER: _ClassVar[int]
    tenant_user_account_status: _tenant_user_account_pb2.TenantUserAccountStatus
    service_account_status: _service_account_pb2.ServiceAccountStatus
    def __init__(self, tenant_user_account_status: _Optional[_Union[_tenant_user_account_pb2.TenantUserAccountStatus, _Mapping]] = ..., service_account_status: _Optional[_Union[_service_account_pb2.ServiceAccountStatus, _Mapping]] = ...) -> None: ...

class GroupMemberKind(_message.Message):
    __slots__ = ["kind"]
    class Kind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        KIND_UNSPECIFIED: _ClassVar[GroupMemberKind.Kind]
        ORDINARY_TENANT_USER_ACCOUNT: _ClassVar[GroupMemberKind.Kind]
        INVITED_TENANT_USER_ACCOUNT: _ClassVar[GroupMemberKind.Kind]
        SERVICE_ACCOUNT: _ClassVar[GroupMemberKind.Kind]
    KIND_UNSPECIFIED: GroupMemberKind.Kind
    ORDINARY_TENANT_USER_ACCOUNT: GroupMemberKind.Kind
    INVITED_TENANT_USER_ACCOUNT: GroupMemberKind.Kind
    SERVICE_ACCOUNT: GroupMemberKind.Kind
    KIND_FIELD_NUMBER: _ClassVar[int]
    kind: GroupMemberKind.Kind
    def __init__(self, kind: _Optional[_Union[GroupMemberKind.Kind, str]] = ...) -> None: ...

class GroupMembershipWithAttributes(_message.Message):
    __slots__ = ["group_membership", "group_member_kind", "user_attributes", "service_account_attributes", "error"]
    GROUP_MEMBERSHIP_FIELD_NUMBER: _ClassVar[int]
    GROUP_MEMBER_KIND_FIELD_NUMBER: _ClassVar[int]
    USER_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    group_membership: GroupMembership
    group_member_kind: GroupMemberKind
    user_attributes: _tenant_user_account_pb2.UserAttributes
    service_account_attributes: _service_account_pb2.ServiceAccountAttributes
    error: _tenant_user_account_pb2.Error
    def __init__(self, group_membership: _Optional[_Union[GroupMembership, _Mapping]] = ..., group_member_kind: _Optional[_Union[GroupMemberKind, _Mapping]] = ..., user_attributes: _Optional[_Union[_tenant_user_account_pb2.UserAttributes, _Mapping]] = ..., service_account_attributes: _Optional[_Union[_service_account_pb2.ServiceAccountAttributes, _Mapping]] = ..., error: _Optional[_Union[_tenant_user_account_pb2.Error, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/lifecycle.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius.storage.v1 import base_pb2 as nebius_dot_storage_dot_v1_dot_base__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/storage/v1/lifecycle.proto\x12\x11nebius.storage.v1\x1a\x1cnebius/storage/v1/base.proto\x1a\x18nebius/annotations.proto\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"[\n\x16LifecycleConfiguration\x12\x41\n\x05rules\x18\x01 \x03(\x0b\x32 .nebius.storage.v1.LifecycleRuleB\t\xbaH\x06\x92\x01\x03\x10\xe8\x07R\x05rules\"\x81\x06\n\rLifecycleRule\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\x12G\n\x06status\x18\x02 \x01(\x0e\x32\'.nebius.storage.v1.LifecycleRule.StatusB\x06\xbaH\x03\xc8\x01\x01R\x06status\x12@\n\x06\x66ilter\x18\x03 \x01(\x0b\x32\".nebius.storage.v1.LifecycleFilterB\x04\xbaJ\x01\x07R\x06\x66ilter\x12\x46\n\nexpiration\x18\x04 \x01(\x0b\x32&.nebius.storage.v1.LifecycleExpirationR\nexpiration\x12{\n\x1dnoncurrent_version_expiration\x18\x05 \x01(\x0b\x32\x37.nebius.storage.v1.LifecycleNoncurrentVersionExpirationR\x1bnoncurrentVersionExpiration\x12\x85\x01\n!abort_incomplete_multipart_upload\x18\x06 \x01(\x0b\x32:.nebius.storage.v1.LifecycleAbortIncompleteMultipartUploadR\x1e\x61\x62ortIncompleteMultipartUpload\x12\x46\n\ntransition\x18\x07 \x01(\x0b\x32&.nebius.storage.v1.LifecycleTransitionR\ntransition\x12{\n\x1dnoncurrent_version_transition\x18\x08 \x01(\x0b\x32\x37.nebius.storage.v1.LifecycleNoncurrentVersionTransitionR\x1bnoncurrentVersionTransition\";\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x0b\n\x07\x45NABLED\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02\"\xab\x01\n\x0fLifecycleFilter\x12\x16\n\x06prefix\x18\x01 \x01(\tR\x06prefix\x12\x42\n\x1eobject_size_greater_than_bytes\x18\x02 \x01(\x03R\x1aobjectSizeGreaterThanBytes\x12<\n\x1bobject_size_less_than_bytes\x18\x03 \x01(\x03R\x17objectSizeLessThanBytes\"\xbb\x01\n\x13LifecycleExpiration\x12\x30\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x04\x64\x61te\x12!\n\x04\x64\x61ys\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01H\x00R\x04\x64\x61ys\x12?\n\x1c\x65xpired_object_delete_marker\x18\x03 \x01(\x08R\x19\x65xpiredObjectDeleteMarkerB\x0e\n\x0c\x65xpired_with\"\xbb\x01\n$LifecycleNoncurrentVersionExpiration\x12?\n\x19newer_noncurrent_versions\x18\x01 \x01(\x05H\x00R\x17newerNoncurrentVersions\x88\x01\x01\x12\x34\n\x0fnoncurrent_days\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x0enoncurrentDaysB\x1c\n\x1a_newer_noncurrent_versions\"j\n\'LifecycleAbortIncompleteMultipartUpload\x12?\n\x15\x64\x61ys_after_initiation\x18\x01 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x13\x64\x61ysAfterInitiation\"\xc2\x01\n\x13LifecycleTransition\x12\x30\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x04\x64\x61te\x12!\n\x04\x64\x61ys\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01H\x00R\x04\x64\x61ys\x12\x44\n\rstorage_class\x18\x03 \x01(\x0e\x32\x1f.nebius.storage.v1.StorageClassR\x0cstorageClassB\x10\n\x0etransited_with\"\x81\x02\n$LifecycleNoncurrentVersionTransition\x12?\n\x19newer_noncurrent_versions\x18\x01 \x01(\x05H\x00R\x17newerNoncurrentVersions\x88\x01\x01\x12\x34\n\x0fnoncurrent_days\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x0enoncurrentDays\x12\x44\n\rstorage_class\x18\x03 \x01(\x0e\x32\x1f.nebius.storage.v1.StorageClassR\x0cstorageClassB\x1c\n\x1a_newer_noncurrent_versionsB]\n\x18\x61i.nebius.pub.storage.v1B\x0eLifecycleProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.lifecycle_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\016LifecycleProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _LIFECYCLECONFIGURATION.fields_by_name['rules']._options = None
  _LIFECYCLECONFIGURATION.fields_by_name['rules']._serialized_options = b'\272H\006\222\001\003\020\350\007'
  _LIFECYCLERULE.fields_by_name['id']._options = None
  _LIFECYCLERULE.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _LIFECYCLERULE.fields_by_name['status']._options = None
  _LIFECYCLERULE.fields_by_name['status']._serialized_options = b'\272H\003\310\001\001'
  _LIFECYCLERULE.fields_by_name['filter']._options = None
  _LIFECYCLERULE.fields_by_name['filter']._serialized_options = b'\272J\001\007'
  _LIFECYCLEEXPIRATION.fields_by_name['days']._options = None
  _LIFECYCLEEXPIRATION.fields_by_name['days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLENONCURRENTVERSIONEXPIRATION.fields_by_name['noncurrent_days']._options = None
  _LIFECYCLENONCURRENTVERSIONEXPIRATION.fields_by_name['noncurrent_days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD.fields_by_name['days_after_initiation']._options = None
  _LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD.fields_by_name['days_after_initiation']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLETRANSITION.fields_by_name['days']._options = None
  _LIFECYCLETRANSITION.fields_by_name['days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLENONCURRENTVERSIONTRANSITION.fields_by_name['noncurrent_days']._options = None
  _LIFECYCLENONCURRENTVERSIONTRANSITION.fields_by_name['noncurrent_days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _globals['_LIFECYCLECONFIGURATION']._serialized_start=174
  _globals['_LIFECYCLECONFIGURATION']._serialized_end=265
  _globals['_LIFECYCLERULE']._serialized_start=268
  _globals['_LIFECYCLERULE']._serialized_end=1037
  _globals['_LIFECYCLERULE_STATUS']._serialized_start=978
  _globals['_LIFECYCLERULE_STATUS']._serialized_end=1037
  _globals['_LIFECYCLEFILTER']._serialized_start=1040
  _globals['_LIFECYCLEFILTER']._serialized_end=1211
  _globals['_LIFECYCLEEXPIRATION']._serialized_start=1214
  _globals['_LIFECYCLEEXPIRATION']._serialized_end=1401
  _globals['_LIFECYCLENONCURRENTVERSIONEXPIRATION']._serialized_start=1404
  _globals['_LIFECYCLENONCURRENTVERSIONEXPIRATION']._serialized_end=1591
  _globals['_LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD']._serialized_start=1593
  _globals['_LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD']._serialized_end=1699
  _globals['_LIFECYCLETRANSITION']._serialized_start=1702
  _globals['_LIFECYCLETRANSITION']._serialized_end=1896
  _globals['_LIFECYCLENONCURRENTVERSIONTRANSITION']._serialized_start=1899
  _globals['_LIFECYCLENONCURRENTVERSIONTRANSITION']._serialized_end=2156
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1/instance.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.compute.v1 import network_interface_pb2 as nebius_dot_compute_dot_v1_dot_network__interface__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n nebius/compute/v1/instance.proto\x12\x11nebius.compute.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\x1a)nebius/compute/v1/network_interface.proto\"\xba\x01\n\x08Instance\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x33\n\x04spec\x18\x02 \x01(\x0b\x32\x1f.nebius.compute.v1.InstanceSpecR\x04spec\x12\x39\n\x06status\x18\x03 \x01(\x0b\x32!.nebius.compute.v1.InstanceStatusR\x06status\"\xb9\x07\n\x0cInstanceSpec\x12\x32\n\x12service_account_id\x18\x01 \x01(\tB\x04\xbaJ\x01\x02R\x10serviceAccountId\x12\x46\n\tresources\x18\x02 \x01(\x0b\x32 .nebius.compute.v1.ResourcesSpecB\x06\xbaH\x03\xc8\x01\x01R\tresources\x12P\n\x0bgpu_cluster\x18\x03 \x01(\x0b\x32).nebius.compute.v1.InstanceGpuClusterSpecB\x04\xbaJ\x01\x02R\ngpuCluster\x12\x63\n\x12network_interfaces\x18\x04 \x03(\x0b\x32\'.nebius.compute.v1.NetworkInterfaceSpecB\x0b\xbaH\x08\xc8\x01\x01\x92\x01\x02\x10\x08R\x11networkInterfaces\x12\x46\n\tboot_disk\x18\x05 \x01(\x0b\x32#.nebius.compute.v1.AttachedDiskSpecB\x04\xbaJ\x01\x02R\x08\x62ootDisk\x12L\n\x0fsecondary_disks\x18\x06 \x03(\x0b\x32#.nebius.compute.v1.AttachedDiskSpecR\x0esecondaryDisks\x12K\n\x0b\x66ilesystems\x18\x07 \x03(\x0b\x32).nebius.compute.v1.AttachedFilesystemSpecR\x0b\x66ilesystems\x12=\n\x14\x63loud_init_user_data\x18\x08 \x01(\tB\x0c\xbaH\x06r\x04\x18\x80\x80\x02\xc0J\x01R\x11\x63loudInitUserData\x12\x18\n\x07stopped\x18\r \x01(\x08R\x07stopped\x12X\n\x0frecovery_policy\x18\x0f \x01(\x0e\x32).nebius.compute.v1.InstanceRecoveryPolicyB\x04\xbaJ\x01\x02R\x0erecoveryPolicy\x12J\n\x0bpreemptible\x18\x13 \x01(\x0b\x32\".nebius.compute.v1.PreemptibleSpecB\x04\xbaJ\x01\x02R\x0bpreemptible\x12\x93\x01\n\x08hostname\x18\x14 \x01(\tBw\xbaHt\xba\x01q\n\x0ehostname.valid\x12\x1evalue must be a valid hostname\x1a?this == \'\' || this.matches(\'^[a-z0-9][a-z0-9-]{0,61}[a-z0-9]$\')R\x08hostname\"\xce\x01\n\x0fPreemptibleSpec\x12\x64\n\ron_preemption\x18\x01 \x01(\x0e\x32\x33.nebius.compute.v1.PreemptibleSpec.PreemptionPolicyB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x0conPreemption\x12&\n\x08priority\x18\x02 \x01(\x05\x42\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x08priority\"-\n\x10PreemptionPolicy\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04STOP\x10\x01\"`\n\rResourcesSpec\x12&\n\x08platform\x18\x01 \x01(\tB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x08platform\x12\x18\n\x06preset\x18\x02 \x01(\tH\x00R\x06presetB\r\n\x04size\x12\x05\xbaH\x02\x08\x01\"(\n\x16InstanceGpuClusterSpec\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xa6\x02\n\x10\x41ttachedDiskSpec\x12W\n\x0b\x61ttach_mode\x18\x01 \x01(\x0e\x32..nebius.compute.v1.AttachedDiskSpec.AttachModeB\x06\xbaH\x03\xc8\x01\x01R\nattachMode\x12\x46\n\rexisting_disk\x18\x02 \x01(\x0b\x32\x1f.nebius.compute.v1.ExistingDiskH\x00R\x0c\x65xistingDisk\x12$\n\tdevice_id\x18\x03 \x01(\tB\x07\xbaH\x04r\x02\x18\x14R\x08\x64\x65viceId\"<\n\nAttachMode\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\r\n\tREAD_ONLY\x10\x01\x12\x0e\n\nREAD_WRITE\x10\x02\x42\r\n\x04type\x12\x05\xbaH\x02\x08\x01\"&\n\x0c\x45xistingDisk\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\",\n\x12\x45xistingFilesystem\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"\xc7\x02\n\x16\x41ttachedFilesystemSpec\x12]\n\x0b\x61ttach_mode\x18\x01 \x01(\x0e\x32\x34.nebius.compute.v1.AttachedFilesystemSpec.AttachModeB\x06\xbaH\x03\xc8\x01\x01R\nattachMode\x12\'\n\tmount_tag\x18\x02 \x01(\tB\n\xbaH\x07\xc8\x01\x01r\x02\x18%R\x08mountTag\x12X\n\x13\x65xisting_filesystem\x18\x03 \x01(\x0b\x32%.nebius.compute.v1.ExistingFilesystemH\x00R\x12\x65xistingFilesystem\"<\n\nAttachMode\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\r\n\tREAD_ONLY\x10\x01\x12\x0e\n\nREAD_WRITE\x10\x02\x42\r\n\x04type\x12\x05\xbaH\x02\x08\x01\"\xa6\x04\n\x0eInstanceStatus\x12\x45\n\x05state\x18\x01 \x01(\x0e\x32/.nebius.compute.v1.InstanceStatus.InstanceStateR\x05state\x12X\n\x12network_interfaces\x18\x02 \x03(\x0b\x32).nebius.compute.v1.NetworkInterfaceStatusR\x11networkInterfaces\x12 \n\x0breconciling\x18\x05 \x01(\x08R\x0breconciling\x12\x30\n\x14maintenance_event_id\x18\x07 \x01(\tR\x12maintenanceEventId\x12s\n\x18infiniband_topology_path\x18\n \x01(\x0b\x32\x37.nebius.compute.v1.InstanceStatusInfinibandTopologyPathH\x00R\x16infinibandTopologyPath\"\x8b\x01\n\rInstanceState\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\x0c\n\x08UPDATING\x10\x02\x12\x0c\n\x08STARTING\x10\x03\x12\x0b\n\x07RUNNING\x10\x04\x12\x0c\n\x08STOPPING\x10\x05\x12\x0b\n\x07STOPPED\x10\x06\x12\x0c\n\x08\x44\x45LETING\x10\x07\x12\t\n\x05\x45RROR\x10\x08\x42\x16\n\x14gpu_cluster_topologyJ\x04\x08\x06\x10\x07\":\n$InstanceStatusInfinibandTopologyPath\x12\x12\n\x04path\x18\x01 \x03(\tR\x04path*/\n\x16InstanceRecoveryPolicy\x12\x0b\n\x07RECOVER\x10\x00\x12\x08\n\x04\x46\x41IL\x10\x01\x42\\\n\x18\x61i.nebius.pub.compute.v1B\rInstanceProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/compute/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1.instance_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.compute.v1B\rInstanceProtoP\001Z/github.com/nebius/gosdk/proto/nebius/compute/v1'
  _INSTANCESPEC.fields_by_name['service_account_id']._options = None
  _INSTANCESPEC.fields_by_name['service_account_id']._serialized_options = b'\272J\001\002'
  _INSTANCESPEC.fields_by_name['resources']._options = None
  _INSTANCESPEC.fields_by_name['resources']._serialized_options = b'\272H\003\310\001\001'
  _INSTANCESPEC.fields_by_name['gpu_cluster']._options = None
  _INSTANCESPEC.fields_by_name['gpu_cluster']._serialized_options = b'\272J\001\002'
  _INSTANCESPEC.fields_by_name['network_interfaces']._options = None
  _INSTANCESPEC.fields_by_name['network_interfaces']._serialized_options = b'\272H\010\310\001\001\222\001\002\020\010'
  _INSTANCESPEC.fields_by_name['boot_disk']._options = None
  _INSTANCESPEC.fields_by_name['boot_disk']._serialized_options = b'\272J\001\002'
  _INSTANCESPEC.fields_by_name['cloud_init_user_data']._options = None
  _INSTANCESPEC.fields_by_name['cloud_init_user_data']._serialized_options = b'\272H\006r\004\030\200\200\002\300J\001'
  _INSTANCESPEC.fields_by_name['recovery_policy']._options = None
  _INSTANCESPEC.fields_by_name['recovery_policy']._serialized_options = b'\272J\001\002'
  _INSTANCESPEC.fields_by_name['preemptible']._options = None
  _INSTANCESPEC.fields_by_name['preemptible']._serialized_options = b'\272J\001\002'
  _INSTANCESPEC.fields_by_name['hostname']._options = None
  _INSTANCESPEC.fields_by_name['hostname']._serialized_options = b'\272Ht\272\001q\n\016hostname.valid\022\036value must be a valid hostname\032?this == \'\' || this.matches(\'^[a-z0-9][a-z0-9-]{0,61}[a-z0-9]$\')'
  _PREEMPTIBLESPEC.fields_by_name['on_preemption']._options = None
  _PREEMPTIBLESPEC.fields_by_name['on_preemption']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _PREEMPTIBLESPEC.fields_by_name['priority']._options = None
  _PREEMPTIBLESPEC.fields_by_name['priority']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _RESOURCESSPEC.oneofs_by_name['size']._options = None
  _RESOURCESSPEC.oneofs_by_name['size']._serialized_options = b'\272H\002\010\001'
  _RESOURCESSPEC.fields_by_name['platform']._options = None
  _RESOURCESSPEC.fields_by_name['platform']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _ATTACHEDDISKSPEC.oneofs_by_name['type']._options = None
  _ATTACHEDDISKSPEC.oneofs_by_name['type']._serialized_options = b'\272H\002\010\001'
  _ATTACHEDDISKSPEC.fields_by_name['attach_mode']._options = None
  _ATTACHEDDISKSPEC.fields_by_name['attach_mode']._serialized_options = b'\272H\003\310\001\001'
  _ATTACHEDDISKSPEC.fields_by_name['device_id']._options = None
  _ATTACHEDDISKSPEC.fields_by_name['device_id']._serialized_options = b'\272H\004r\002\030\024'
  _EXISTINGDISK.fields_by_name['id']._options = None
  _EXISTINGDISK.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _EXISTINGFILESYSTEM.fields_by_name['id']._options = None
  _EXISTINGFILESYSTEM.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _ATTACHEDFILESYSTEMSPEC.oneofs_by_name['type']._options = None
  _ATTACHEDFILESYSTEMSPEC.oneofs_by_name['type']._serialized_options = b'\272H\002\010\001'
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['attach_mode']._options = None
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['attach_mode']._serialized_options = b'\272H\003\310\001\001'
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['mount_tag']._options = None
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['mount_tag']._serialized_options = b'\272H\007\310\001\001r\002\030%'
  _globals['_INSTANCERECOVERYPOLICY']._serialized_start=3006
  _globals['_INSTANCERECOVERYPOLICY']._serialized_end=3053
  _globals['_INSTANCE']._serialized_start=187
  _globals['_INSTANCE']._serialized_end=373
  _globals['_INSTANCESPEC']._serialized_start=376
  _globals['_INSTANCESPEC']._serialized_end=1329
  _globals['_PREEMPTIBLESPEC']._serialized_start=1332
  _globals['_PREEMPTIBLESPEC']._serialized_end=1538
  _globals['_PREEMPTIBLESPEC_PREEMPTIONPOLICY']._serialized_start=1493
  _globals['_PREEMPTIBLESPEC_PREEMPTIONPOLICY']._serialized_end=1538
  _globals['_RESOURCESSPEC']._serialized_start=1540
  _globals['_RESOURCESSPEC']._serialized_end=1636
  _globals['_INSTANCEGPUCLUSTERSPEC']._serialized_start=1638
  _globals['_INSTANCEGPUCLUSTERSPEC']._serialized_end=1678
  _globals['_ATTACHEDDISKSPEC']._serialized_start=1681
  _globals['_ATTACHEDDISKSPEC']._serialized_end=1975
  _globals['_ATTACHEDDISKSPEC_ATTACHMODE']._serialized_start=1900
  _globals['_ATTACHEDDISKSPEC_ATTACHMODE']._serialized_end=1960
  _globals['_EXISTINGDISK']._serialized_start=1977
  _globals['_EXISTINGDISK']._serialized_end=2015
  _globals['_EXISTINGFILESYSTEM']._serialized_start=2017
  _globals['_EXISTINGFILESYSTEM']._serialized_end=2061
  _globals['_ATTACHEDFILESYSTEMSPEC']._serialized_start=2064
  _globals['_ATTACHEDFILESYSTEMSPEC']._serialized_end=2391
  _globals['_ATTACHEDFILESYSTEMSPEC_ATTACHMODE']._serialized_start=1900
  _globals['_ATTACHEDFILESYSTEMSPEC_ATTACHMODE']._serialized_end=1960
  _globals['_INSTANCESTATUS']._serialized_start=2394
  _globals['_INSTANCESTATUS']._serialized_end=2944
  _globals['_INSTANCESTATUS_INSTANCESTATE']._serialized_start=2775
  _globals['_INSTANCESTATUS_INSTANCESTATE']._serialized_end=2914
  _globals['_INSTANCESTATUSINFINIBANDTOPOLOGYPATH']._serialized_start=2946
  _globals['_INSTANCESTATUSINFINIBANDTOPOLOGYPATH']._serialized_end=3004
# @@protoc_insertion_point(module_scope)



# type: ignore
import logging

import pytest

from nebius.aio import request

request.DEFAULT_AUTH_TIMEOUT = 5.0


@pytest.mark.asyncio
async def test_get_instance_sync_in_async_no_loop() -> None:
    import grpc
    import grpc.aio

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.compute.v1.disk_pb2 as disk_pb2
    from nebius.aio.channel import Channel, LoopError, NoCredentials
    from nebius.api.nebius.compute.v1.disk_service_pb2 import (
        GetDiskRequest,
    )
    from nebius.api.nebius.compute.v1.disk_service_pb2_grpc import (
        DiskServiceServicer,
        add_DiskServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(DiskServiceServicer):
        async def Get(  # noqa: N802 — GRPC method
            self,
            request: GetDiskRequest,
            context: grpc.aio.ServicerContext[GetDiskRequest, disk_pb2.Disk],
        ) -> disk_pb2.Disk:
            assert request.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""

            # Return an Instance object as expected by the client
            ret = disk_pb2.Disk()
            ret.metadata.id = request.id
            ret.metadata.name = "MockDisk"
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_DiskServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    try:
        # Set up the client channel
        channel = Channel(
            domain=address,
            options=[(INSECURE, True)],
            credentials=NoCredentials(),
        )
        from nebius.api.nebius.compute.v1 import DiskServiceClient, GetDiskRequest

        client = DiskServiceClient(channel)
        req = client.get(GetDiskRequest(id="foo-bar"))

        # Await response and metadata
        req.wait()
    except LoopError as e:
        assert (
            str(e) == "Synchronous call inside async context. Either use async/"
            "await or provide a safe and separate loop to run at the SDK "
            "initialization."
        )
    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)


@pytest.mark.asyncio
async def test_get_instance_sync_in_async_same_loop() -> None:
    from asyncio import (
        get_event_loop,
    )

    import grpc
    import grpc.aio

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.compute.v1.disk_pb2 as disk_pb2
    from nebius.aio.channel import Channel, LoopError, NoCredentials
    from nebius.api.nebius.compute.v1.disk_service_pb2 import (
        GetDiskRequest,
    )
    from nebius.api.nebius.compute.v1.disk_service_pb2_grpc import (
        DiskServiceServicer,
        add_DiskServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(DiskServiceServicer):
        async def Get(  # noqa: N802 — GRPC method
            self,
            request: GetDiskRequest,
            context: grpc.aio.ServicerContext[GetDiskRequest, disk_pb2.Disk],
        ) -> disk_pb2.Disk:
            assert request.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""

            # Return an Instance object as expected by the client
            ret = disk_pb2.Disk()
            ret.metadata.id = request.id
            ret.metadata.name = "MockDisk"
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_DiskServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    try:
        # Set up the client channel
        channel = Channel(
            domain=address,
            options=[(INSECURE, True)],
            event_loop=get_event_loop(),
            credentials=NoCredentials(),
        )
        from nebius.api.nebius.compute.v1 import DiskServiceClient, GetDiskRequest

        client = DiskServiceClient(channel)
        req = client.get(GetDiskRequest(id="foo-bar"))

        # Await response and metadata
        req.wait()
    except LoopError as e:
        assert (
            str(e) == "Provided loop is equal to current thread's loop. Either use "
            "async/await or provide another loop at the SDK initialization."
        )
    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)


@pytest.mark.asyncio
async def test_get_instance_v2() -> None:
    import grpc
    import grpc.aio

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.compute.v1.disk_pb2 as disk_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.api.nebius.compute.v1.disk_service_pb2 import (
        GetDiskRequest,
    )
    from nebius.api.nebius.compute.v1.disk_service_pb2_grpc import (
        DiskServiceServicer,
        add_DiskServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(DiskServiceServicer):
        async def Get(  # noqa: N802 — GRPC method
            self,
            request: GetDiskRequest,
            context: grpc.aio.ServicerContext[GetDiskRequest, disk_pb2.Disk],
        ) -> disk_pb2.Disk:
            assert request.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""

            await context.send_initial_metadata(
                (
                    ("x-request-id", "some-req-id"),
                    ("x-trace-id", "some-trace-id"),
                )
            )

            # Return an Instance object as expected by the client
            ret = disk_pb2.Disk()
            ret.metadata.id = request.id
            ret.metadata.name = "MockDisk"
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_DiskServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    from nebius.aio import request

    request.DEFAULT_AUTH_TIMEOUT = 5.0
    try:
        # Set up the client channel
        channel = Channel(
            domain=address,
            options=[(INSECURE, True)],
            credentials=NoCredentials(),
        )
        from nebius.api.nebius.compute.v1 import Disk, DiskServiceClient, GetDiskRequest

        client = DiskServiceClient(channel)
        req = client.get(GetDiskRequest(id="foo-bar"))

        # Await response and metadata
        ret = await req
        assert isinstance(ret, Disk)
        # Assertions to validate behavior
        assert ret.metadata.id == "foo-bar"
        assert ret.metadata.name == "MockDisk"

    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)


@pytest.mark.asyncio
async def test_status_not_blocks_get_instance_v2() -> None:
    import grpc
    import grpc.aio
    from grpc import StatusCode

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.compute.v1.disk_pb2 as disk_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.api.nebius.compute.v1.disk_service_pb2 import (
        GetDiskRequest,
    )
    from nebius.api.nebius.compute.v1.disk_service_pb2_grpc import (
        DiskServiceServicer,
        add_DiskServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(DiskServiceServicer):
        async def Get(  # noqa: N802 — GRPC method
            self,
            request: GetDiskRequest,
            context: grpc.aio.ServicerContext[GetDiskRequest, disk_pb2.Disk],
        ) -> disk_pb2.Disk:
            assert request.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""

            await context.send_initial_metadata(
                (
                    ("x-request-id", "some-req-id"),
                    ("x-trace-id", "some-trace-id"),
                )
            )

            # Return an Instance object as expected by the client
            ret = disk_pb2.Disk()
            ret.metadata.id = request.id
            ret.metadata.name = "MockDisk"
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_DiskServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    try:
        # Set up the client channel
        channel = Channel(
            domain=address, options=[(INSECURE, True)], credentials=NoCredentials()
        )
        from nebius.api.nebius.compute.v1 import Disk, DiskServiceClient, GetDiskRequest

        client = DiskServiceClient(channel)
        req = client.get(GetDiskRequest(id="foo-bar"))

        status_coro = req.status()
        # Await response and metadata
        ret = await req
        assert isinstance(ret, Disk)
        status = await status_coro
        assert status.code == StatusCode.OK
        # Assertions to validate behavior
        assert ret.metadata.id == "foo-bar"
        assert ret.metadata.name == "MockDisk"

    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class StorageClass(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    STORAGE_CLASS_UNSPECIFIED: _ClassVar[StorageClass]
    STANDARD: _ClassVar[StorageClass]
    ENHANCED_THROUGHPUT: _ClassVar[StorageClass]

class VersioningPolicy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    VERSIONING_POLICY_UNSPECIFIED: _ClassVar[VersioningPolicy]
    DISABLED: _ClassVar[VersioningPolicy]
    ENABLED: _ClassVar[VersioningPolicy]
    SUSPENDED: _ClassVar[VersioningPolicy]
STORAGE_CLASS_UNSPECIFIED: StorageClass
STANDARD: StorageClass
ENHANCED_THROUGHPUT: StorageClass
VERSIONING_POLICY_UNSPECIFIED: VersioningPolicy
DISABLED: VersioningPolicy
ENABLED: VersioningPolicy
SUSPENDED: VersioningPolicy



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/session_management_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.nebius/iam/v1/session_management_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\"\xad\x01\n\x14RevokeSessionRequest\x12.\n\x12service_account_id\x18\x03 \x01(\tH\x00R\x10serviceAccountId\x12$\n\rall_my_active\x18\x06 \x01(\x08H\x00R\x0b\x61llMyActive\x12\x35\n\x16tenant_user_account_id\x18\x07 \x01(\tH\x00R\x13tenantUserAccountIdB\x08\n\x06revoke\"\x17\n\x15RevokeSessionResponse2{\n\x18SessionManagementService\x12S\n\x06Revoke\x12#.nebius.iam.v1.RevokeSessionRequest\x1a$.nebius.iam.v1.RevokeSessionResponse\x1a\n\xbaJ\x07\x63pl.iamBd\n\x14\x61i.nebius.pub.iam.v1B\x1dSessionManagementServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.session_management_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\035SessionManagementServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _SESSIONMANAGEMENTSERVICE._options = None
  _SESSIONMANAGEMENTSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_REVOKESESSIONREQUEST']._serialized_start=92
  _globals['_REVOKESESSIONREQUEST']._serialized_end=265
  _globals['_REVOKESESSIONRESPONSE']._serialized_start=267
  _globals['_REVOKESESSIONRESPONSE']._serialized_end=290
  _globals['_SESSIONMANAGEMENTSERVICE']._serialized_start=292
  _globals['_SESSIONMANAGEMENTSERVICE']._serialized_end=415
# @@protoc_insertion_point(module_scope)
